
//------------------------------------------------------------------------ check paper sensor (returns true for paper, false for no paper)
bool checkPaperSensor(){
  return (analogRead(PAPER_SENSOR_1)<SENSOR_THRESHOLD && analogRead(PAPER_SENSOR_2)<SENSOR_THRESHOLD);
}

//------------------------------------------------------------------------ checkCoin
unsigned long lastCoinTime = 0;
bool checkCoin(){
  bool gotCoin = false;
  if (digitalRead(COIN_SWITCH) && lastCoinTime+500<millis()){
    gotCoin = true;
    lastCoinTime = millis();
  }
  if (gotCoin) Serial.println("COIN");
  return gotCoin;
}

//------------------------------------------------------------------------ move motor
void motor(boolean motOnOff) {
  if (motOnOff) {
    analogWrite(TENSION_MOT, 150);
  } else {
    digitalWrite(TENSION_MOT, LOW);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////// plot text

void plotText(String str, int x, int y, int scale, int space) {
  int pos = 0;
  // Serial.println("plot string");
  // Serial.println(str);
  for (int i = 0; i < str.length(); i++) {
    char c = char(str.charAt(i));
    if (byte(c) != 195) {
      if (c == ' ') {
        pos += scale;
      } else {
        plotChar(c, x + pos, y, scale);
        pos = pos + space;
        if (c == 'I' || c == 'i') pos -= scale / 1.1;
        if (c == ',') pos -= scale / 1.2;
      }
    }
  }
  // Serial.println();
  releaseMotors();
}



////////////////////////////////////////////////////////////////////////////////////////////////////// plot character at x,y position

void plotChar(char c, int x, int y, int scale) {
  scale /= 4;
  uint8_t character = 38;
  if (uint8_t(c) > 64 and uint8_t(c) < 91) {  //A...Z
    character = uint8_t(c) - 65;
  }
  if (uint8_t(c) > 96 and uint8_t(c) < 123) {  //A...Z
    character = uint8_t(c) - 97;
  }
  if (uint8_t(c) > 47 and uint8_t(c) < 58) {  //0...9
    character = uint8_t(c) - 22;
  }
  if (uint8_t(c) == 164 || uint8_t(c) == 132) {  //ä,Ä
    character = 39;
  }
  if (uint8_t(c) == 182 || uint8_t(c) == 150) {  //ö,Ö
    character = 40;
  }
  if (uint8_t(c) == 188 || uint8_t(c) == 156) {  //ü,Ü
    character = 41;
  }
  if (uint8_t(c) == 44) {  // ,
    character = 42;
  }
  if (uint8_t(c) == 45) {  // -
    character = 43;
  }
  if (uint8_t(c) == 46) {  // .
    character = 44;
  }
  if (uint8_t(c) == 33) {  // !
    character = 45;
  }
  if (uint8_t(c) == 63) {  // ?
    character = 46;
  }

  if (uint8_t(c) == 123) { /*{ ß*/
    character = 47;
  }
  if (uint8_t(c) == 39) { /*'*/
    character = 48;
  }
  if (uint8_t(c) == 38) { /*&*/
    character = 49;
  }
  if (uint8_t(c) == 43) { /*+*/
    character = 50;
  }
  if (uint8_t(c) == 58) { /*:*/
    character = 51;
  }
  if (uint8_t(c) == 59) { /*;*/
    character = 52;
  }
  if (uint8_t(c) == 34) { /*"*/
    character = 53;
  }
  if (uint8_t(c) == 35) { /*#*/
    character = 54;
  }
  if (uint8_t(c) == 40) { /*(*/
    character = 55;
  }
  if (uint8_t(c) == 41) { /*)*/
    character = 56;
  }
  if (uint8_t(c) == 61) { /*=*/
    character = 57;
  }
  if (uint8_t(c) == 64) { /*@*/
    character = 58;
  }
  if (uint8_t(c) == 42) { /***/
    character = 59;
  }
  if (uint8_t(c) == 125) { /*} Smiley*/
    character = 60;
  }
  if (uint8_t(c) == 126) { /*~ Open mouth Smiley*/
    character = 61;
  }
  if (uint8_t(c) == 36) { /*$ Heart*/
    character = 62;
  }

  //Serial.println(character);
  for (int i = 0; i < 14; i++) {  // go through each vector of the character    Serial.print("|| v:");

    int v = vector[character][i];
    if (v == 200) {  // no more vectors in this array
      break;
    }
    if (v == 222) {  // plot single point
      plot(true);
      plot(false);
    } else {
      int draw = 0;
      if (v > 99) {
        draw = 1;
        v -= 100;
      }
      int cx = v / 10;       // get y ...
      int cy = v - cx * 10;  // and x
      moveTo((x + cx * scale), (4 - cy) * scale + y, draw);
    }
  }
}


////////////////////////////////////////////////////////////////////////////////////////////////////// plot

void plot(boolean penOnPaper) {
  digitalWrite(PEN, penOnPaper);
  fans(penOnPaper);
  if (penOnPaper != pPenOnPaper) delay(100);
  pPenOnPaper = penOnPaper;
}

////////////////////////////////////////////////////////////////////////////////////////////////////// fans
void fans(bool onOff) {
  digitalWrite(FANS, onOff);
}

////////////////////////////////////////////////////////////////////////////////////////////////////// line

void line(float xa, float ya, float xb, float yb) {
  // move to starting position
  moveTo(xa, ya, 0);

  // move to end position
  moveTo(xb, yb, 1);
}

////////////////////////////////////////////////////////////////////////////////////////////////////// y-feed
boolean yFeed(int steps) {
  ypos += steps;
}


////////////////////////////////////////////////////////////////////////////////////////////////////// moveTo

// ramp speeds
int stdDuration = 220;  // maximum delay in ms: applies at beginning and end of move
int maxRamp = 160;      // delay reduction during move; value in 1/2ms

boolean moveTo(float xb, float yb, boolean stroke) {
  motor(1);
  xb = constrain(xb, 0, XMAX);
  float xa = xpos;
  float ya = ypos;

  float difX = abs(xa - xb);
  float difY = abs(ya - yb);

  int rampCount = 0;
  int ramp = 0;
  int cycles = difX;
  if (cycles < difY) cycles = difY;
  if (difX >= 1 || difY >= 1) plot(stroke);


  // move to b-position

  if (difX > difY || difY == 0 || difX == difY) {

    while (abs(xa - xb) >= 1 || abs(ya - yb) >= 1) {  // while position is wrong
      // calculate ramps
      if (ramp < maxRamp) ramp++;                                // ramp slope at beginning until maxRamp is reached
      if (ramp > cycles - rampCount) ramp = cycles - rampCount;  // ramp slope at end
      rampCount++;
      int delayTime = stdDuration - ramp / 2;

      if (xb < xa) xa--;
      if (xb > xa) xa++;
      if (yb < ya) ya -= abs(difY / difX);
      if (yb > ya) ya += abs(difY / difX);
      xpos = int(xa + .5);
      ypos = int(ya + .5);
      for (int i = 0; i < 4; i++) {
        digitalWrite(xPins[i], halfStep[(xpos % 8 + 8) % 8][i]);
        digitalWrite(yPins[i], halfStep[(ypos % 8 + 8) % 8][i]);
        delayMicroseconds(delayTime);
      }
    }
  }

  if (difX < difY || difX == 0) {
    while (abs(xa - xb) >= 1 || abs(ya - yb) >= 1) {
      // calculate ramps
      if (ramp < maxRamp) ramp++;                                // ramp slope at beginning until maxRamp is reached
      if (ramp > cycles - rampCount) ramp = cycles - rampCount;  // ramp slope at end
      rampCount++;
      int delayTime = stdDuration - ramp / 2;

      if (yb < ya) ya--;
      if (yb > ya) ya++;
      if (xb < xa) xa -= abs(difX / difY);
      if (xb > xa) xa += abs(difX / difY);
      xpos = int(xa + .5);
      ypos = int(ya + .5);
      for (int i = 0; i < 4; i++) {
        digitalWrite(xPins[i], halfStep[(xpos % 8 + 8) % 8][i]);
        digitalWrite(yPins[i], halfStep[(ypos % 8 + 8) % 8][i]);
        delayMicroseconds(delayTime);
      }
    }
  }
  motor(0);
}



////////////////////////////////////////////////////////////////////////////////////////////////////// release stepper motors

void releaseMotors() {
  for (int i = 0; i < 4; i++) {
    digitalWrite(xPins[i], 0);
    digitalWrite(yPins[i], 0);
  }
  plot(false);
}
////////////////////////////////////////////////////////////////////////////////////////////////////// stamp
bool stamp() {  // stamping the paper - returns true for stamping, false for error
  Serial.println("Stamping");
  boolean returnValue = true;
  unsigned long startTime = millis();
  unsigned long timeOut = 2000;  // predefined timeout



  while (!digitalRead(STAMP_SWITCH) && returnValue) { // stamp on paper
    analogWrite(STAMP_MOT, 200);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(STAMP_MOT, LOW);
  
    Serial.print("Stamp time: ");
  Serial.print(millis() - startTime);
  Serial.print(" Timeout: ");
  Serial.println(timeOut);

  delay (500);
  startTime = millis();
    while (digitalRead(STAMP_SWITCH) && returnValue) {  // return stamp
    analogWrite(STAMP_MOT, 120);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(STAMP_MOT, LOW);

    Serial.print("Stamp release time: ");
  Serial.print(millis() - startTime);
  Serial.print(" Timeout: ");
  Serial.println(timeOut);
  
  return returnValue;
}
////////////////////////////////////////////////////////////////////////////////////////////////////// tighten ruler
bool tightenRuler() {  // tighten the ruler for cutting
  Serial.println("Tighten ruler");
  boolean returnValue = true;
  unsigned long startTime = millis();
  unsigned long timeOut = 2000;  // predefined timeout
  while (digitalRead(RULER_SWITCH) && returnValue) {
    analogWrite(RULER_MOT, 80);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(RULER_MOT, LOW);
    Serial.print("Tighten time: ");
  Serial.print(millis() - startTime);
  Serial.print(" Timeout: ");
  Serial.println(timeOut);
  return returnValue;
}
////////////////////////////////////////////////////////////////////////////////////////////////////// release ruler
bool releaseRuler() {  // release ruler
  Serial.println("Release ruler");
  boolean returnValue = true;
  unsigned long startTime = millis();
  unsigned long timeOut = 2000;  // predefined timeout

  while (!digitalRead(RULER_SWITCH) && returnValue) {
    analogWrite(RULER_MOT, 80);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(RULER_MOT, LOW);
  Serial.print("Release time: ");
  Serial.print(millis() - startTime);
  Serial.print(" Timeout: ");
  Serial.println(timeOut);
  return returnValue;
}


////////////////////////////////////////////////////////////////////////////////////////////////////// Cut paper
bool cutPaper() {  // cuts off paper - returns true for success and false for failure
  tightenRuler();
  Serial.println("Cut paper");
  unsigned long startTime = millis();
  unsigned long timeOut = 10000;  // predefined timeouts
  boolean returnValue = true;

  while (digitalRead(CUTTER_SWITCH_L) && returnValue) {  // move cutter left (cut)
    analogWrite(CUTTER_MOT_A, 135);                      // left
    digitalWrite(CUTTER_MOT_B, LOW);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(CUTTER_MOT_A, HIGH);  // motor break
  digitalWrite(CUTTER_MOT_B, HIGH);

  if (returnValue) {
    Serial.print("Cutter moved left: ");
    Serial.print(millis() - startTime);
    Serial.print(" Timeout: ");
    Serial.println(timeOut);
  }

  delay(100);
  digitalWrite(CUTTER_MOT_A, LOW);  // stop
  digitalWrite(CUTTER_MOT_B, LOW);
  if (returnValue) { returnValue = returnCutter(); }
  return returnValue;
}

bool returnCutter() {  //
  Serial.println("Return cutter");
  unsigned long startTime = millis();
  unsigned long timeOut = 10000;  // predefined timeouts
  boolean returnValue = true;
  timeOut = 10000;
  startTime = millis();
  while (digitalRead(CUTTER_SWITCH_R) && returnValue) {  // return cutter
    analogWrite(CUTTER_MOT_B, 150);                      // right
    digitalWrite(CUTTER_MOT_A, LOW);
    if (millis() - startTime > timeOut) {
      returnValue = false;
    }
  }
  digitalWrite(CUTTER_MOT_A, HIGH);  // motor break
  digitalWrite(CUTTER_MOT_B, HIGH);

  if (returnValue) {
    Serial.print("Cutter moved right: ");
    Serial.print(millis() - startTime);
    Serial.print(" Timeout: ");
    Serial.println(timeOut);
  } else {
    Serial.println("Cutting abortet!");
  }

  delay(100);
  digitalWrite(CUTTER_MOT_A, LOW);  // stop
  digitalWrite(CUTTER_MOT_B, LOW);
  if (returnValue) { returnValue = releaseRuler(); }
  return returnValue;
}
////////////////////////////////////////////////////////////////////////////////////////////////////// calibrate

boolean calibrate() {
  Serial.println("Calibrating Plotter ...");
  digitalWrite(PEN, false);
  xpos = 20000;
  while (!digitalRead(PLOTTER_END_SWITCH)) {
    xpos--;
    for (int i = 0; i < 4; i++) {
      digitalWrite(xPins[i], halfStep[xpos % 8][i]);
      delayMicroseconds(190);
    }
  }

  for (int i = 0; i < 20; i++) {
    xpos++;
    for (int i = 0; i < 4; i++) {
      digitalWrite(xPins[i], halfStep[xpos % 8][i]);
      delayMicroseconds(190);
    }
  }
  releaseMotors();
  Serial.println("Plotter Calibrated!");
  xpos = 0;
  ypos = defaultY;
  return true;
}
